@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: file.c 15598 2013-11-11 12:33:32Z yeti-dn $
+ *  @(#) $Id: file.c 17755 2015-11-14 09:19:43Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -309,14 +309,30 @@ gwy_app_file_load_real(const gchar *filename_utf8,
     return data;
 }
 
+static void
+set_filename(GwyContainer *data, const gchar *filename, gboolean take)
+{
+    GQuark quark = g_quark_from_string("/filename");
+
+    if (gwy_container_contains(data, quark)) {
+        if (take)
+            g_free((gpointer)filename);
+        return;
+    }
+
+    if (take)
+        gwy_container_set_string(data, quark, (const guchar*)filename);
+    else
+        gwy_container_set_const_string(data, quark, (const guchar*)filename);
+}
+
 static void
 gwy_app_file_add_loaded(GwyContainer *data,
                         const gchar *filename_utf8,
                         const gchar *filename_sys)
 {
     if (filename_utf8)
-        gwy_container_set_string_by_name(data, "/filename",
-                                         g_strdup(filename_utf8));
+        set_filename(data, filename_utf8, FALSE);
 
     gwy_app_data_browser_add(data);
     gwy_app_data_browser_reset_visibility(data,
@@ -475,12 +491,11 @@ gwy_app_file_write(GwyContainer *data,
         case GWY_FILE_OPERATION_SAVE:
         if (free_utf8) {
             gwy_undo_container_set_unmodified(data);
-            gwy_container_set_string_by_name(data, "/filename", filename_utf8);
+            set_filename(data, filename_utf8, TRUE);
             free_utf8 = FALSE;
         }
         else
-            gwy_container_set_string_by_name(data, "/filename",
-                                             g_strdup(filename_utf8));
+            set_filename(data, filename_utf8, FALSE);
         gwy_app_recent_file_list_update(data, filename_utf8, filename_sys, id);
 
         case GWY_FILE_OPERATION_EXPORT:
