@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: filelist.c 17365 2015-09-13 06:52:22Z yeti-dn $
+ *  @(#) $Id: filelist.c 17718 2015-11-06 08:04:04Z pecold $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -31,7 +31,6 @@
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -796,7 +795,7 @@ gwy_app_recent_file_list_save(const gchar *filename)
     FILE *fh;
 
     g_return_val_if_fail(gcontrols.store, FALSE);
-    fh = g_fopen(filename, "w");
+    fh = gwy_fopen(filename, "w");
     if (!fh)
         return FALSE;
 
