@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwysiunit.c 16871 2014-12-26 11:16:39Z yeti-dn $
+ *  @(#) $Id: gwysiunit.c 17581 2015-10-17 09:57:39Z dn2010 $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -127,7 +127,7 @@ SI_prefixes[] = {
 
 /* Units that can conflict with prefixes */
 static const gchar *known_units[] = {
-    "deg", "Pa", "cd", "mol", "cal", "px", "pt", "cps",
+    "deg", "Pa", "cd", "mol", "cal", "px", "pt", "cps", "cts",
 };
 
 /* Long names.  Keep the list sorted by length so that we can give up quickly
@@ -999,6 +999,9 @@ gwy_si_unit_parse(GwySIUnit *siunit,
             pfpower -= 2;
             g_string_assign(buf, "");
         }
+        else if (gwy_strequal(buf->str, "м")) {
+            g_string_assign(buf, "m");
+        }
 
         /* elementary sanity */
         if (!g_utf8_validate(buf->str, -1, (const gchar**)&p)) {
