@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyprocessenums.h 17356 2015-09-11 20:02:09Z yeti-dn $
+ *  @(#) $Id: gwyprocessenums.h 17567 2015-10-16 10:05:56Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -243,8 +243,21 @@ typedef enum {
     GWY_DISTANCE_TRANSFORM_CONN8       = 1,
     GWY_DISTANCE_TRANSFORM_OCTAGONAL48 = 2,
     GWY_DISTANCE_TRANSFORM_OCTAGONAL84 = 3,
+    GWY_DISTANCE_TRANSFORM_OCTAGONAL   = 4,
+    GWY_DISTANCE_TRANSFORM_EUCLIDEAN   = 5,
 } GwyDistanceTransformType;
 
+typedef enum {
+    GWY_MIN_MAX_FILTER_MINIMUM       = 0,
+    GWY_MIN_MAX_FILTER_EROSION       = 0,
+    GWY_MIN_MAX_FILTER_MAXIMUM       = 1,
+    GWY_MIN_MAX_FILTER_DILATION      = 1,
+    GWY_MIN_MAX_FILTER_OPENING       = 2,
+    GWY_MIN_MAX_FILTER_CLOSING       = 3,
+    GWY_MIN_MAX_FILTER_RANGE         = 4,
+    GWY_MIN_MAX_FILTER_NORMALIZATION = 5,
+} GwyMinMaxFilterType;
+
 const GwyEnum* gwy_merge_type_get_enum(void) G_GNUC_CONST;
 const GwyEnum* gwy_masking_type_get_enum(void) G_GNUC_CONST;
 const GwyEnum* gwy_plane_symmetry_get_enum(void) G_GNUC_CONST;
@@ -255,6 +268,7 @@ const GwyEnum* gwy_dwt_denoise_type_get_enum(void) G_GNUC_CONST;
 const GwyEnum* gwy_interpolation_type_get_enum(void) G_GNUC_CONST;
 const GwyEnum* gwy_windowing_type_get_enum(void) G_GNUC_CONST;
 const GwyEnum* gwy_correlation_type_get_enum(void) G_GNUC_CONST;
+const GwyEnum* gwy_distance_transform_type_get_enum(void) G_GNUC_CONST;
 
 /* Well, this is not exactly an enum... */
 typedef struct {
