@@ -1,5 +1,5 @@
 /*
- *  $Id: gxyzffile.c 17138 2015-05-10 06:17:15Z yeti-dn $
+ *  $Id: gxyzffile.c 17777 2015-11-23 14:35:11Z yeti-dn $
  *  Copyright (C) 2013 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -46,7 +46,6 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <glib/gstdio.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
@@ -111,7 +110,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Gwyddion XYZ field files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.2",
+    "1.3",
     "David Nečas (Yeti)",
     "2013",
 };
@@ -326,7 +325,7 @@ fail:
         g_free(dfields);
     }
     if (grid) {
-        for (i = 0; i < npoints; i++)
+        for (i = 0; i < ginfo.xres*ginfo.yres; i++)
             g_slist_free(grid[i]);
         g_free(grid);
     }
@@ -376,7 +375,7 @@ gxyzf_export(GwyContainer *container,
         return FALSE;
     }
 
-    if (!(fh = g_fopen(filename, "wb"))) {
+    if (!(fh = gwy_fopen(filename, "wb"))) {
         err_OPEN_WRITE(error);
         return FALSE;
     }
