@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoscantech.c 17513 2015-10-06 11:02:18Z pecold $
+ *  $Id: nanoscantech.c 17718 2015-11-06 08:04:04Z pecold $
  *  Copyright (C) 2012 David Necas (Yeti), Daniil Bratashov (dn2010).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,8 @@
 
 /*
  * TODO: assuming cp1251 as 8bit encoding,
- *       use Attributes field to load parameters
+ *       use Attributes field to load 3d xyz labels,
+ *       4d xy ranges/units/labels;
  *       4d data loading improvements for jumping mode
  */
 
@@ -51,10 +52,7 @@
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <limits.h>
-#include <unzip.h>
-#include <glib/gstdio.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libgwyddion/gwyutils.h>
@@ -63,15 +61,7 @@
 #include <app/data-browser.h>
 
 #include "err.h"
-#include "gwyminizip.h"
-
-#ifdef _MSC_VER
-    #include <windows.h>
-    #ifdef PATH_MAX
-    #undef PATH_MAX
-    #endif
-    #define PATH_MAX MAX_PATH
-#endif
+#include "gwyzip.h"
 
 #define MAGIC "PK\x03\x04"
 #define MAGIC_SIZE (sizeof(MAGIC)-1)
@@ -128,17 +118,14 @@ static GwyBrick*      nst_read_4d         (const gchar *buffer,
                                            gsize datasize,
                                            GwyContainer **metadata,
                                            gchar **title);
-static guchar*        nst_get_file_content(unzFile *zipfile,
-                                           gsize *contentsize,
-                                           GError **error);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports NanoScanTech .nstdat files."),
     "Daniil Bratashov (dn2010@gmail.com)",
-    "0.8",
-    "David Nečas (Yeti), Daniil Bratashov (dn2010)",
+    "0.10",
+    "David Nečas (Yeti), Daniil Bratashov (dn2010), Antony Kikaxa",
     "2012",
 };
 
@@ -161,7 +148,7 @@ static gint
 nst_detect(const GwyFileDetectInfo *fileinfo,
            gboolean only_name)
 {
-    unzFile zipfile;
+    GwyZipFile zipfile;
 
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION) ? 15 : 0;
@@ -171,21 +158,21 @@ nst_detect(const GwyFileDetectInfo *fileinfo,
         || memcmp(fileinfo->head, MAGIC, MAGIC_SIZE) != 0)
         return 0;
 
-    /* It contains directory Scan so this should be somewhere near the begining
-     * of the file. */
+    /* It contains directory Scan so this should be somewhere near
+     * the begining of the file. */
     if (!gwy_memmem(fileinfo->head, fileinfo->buffer_len, MAGIC1, MAGIC1_SIZE))
         return 0;
 
     /* We have to realy look inside. */
-    if (!(zipfile = gwyminizip_unzOpen(fileinfo->name)))
+    if (!(zipfile = gwyzip_open(fileinfo->name)))
         return 0;
 
-    if (unzLocateFile(zipfile, "0.lsdlsd", 1) != UNZ_OK) {
-        unzClose(zipfile);
+    if (!gwyzip_locate_file(zipfile, "0.lsdlsd", 1, NULL)) {
+        gwyzip_close(zipfile);
         return 0;
     }
 
-    unzClose(zipfile);
+    gwyzip_close(zipfile);
 
     return 100;
 }
@@ -199,14 +186,15 @@ nst_load(const gchar *filename,
     GwyDataField *dfield;
     GwyGraphModel *gmodel;
     GwyBrick *brick;
-    unzFile zipfile;
+    GwyZipFile zipfile;
     guint channelno = 0;
-    gint status, xres, yres;
-    gchar *buffer, *line, *p, *title, *strkey;
+    gint xres, yres;
+    gboolean status;
+    gchar *buffer, *line, *p, *title, *strkey, *filename_curr;
     gchar *titlestr = NULL;
     gsize size = 0;
 
-    zipfile = gwyminizip_unzOpen(filename);
+    zipfile = gwyzip_open(filename);
     if (!zipfile) {
         g_set_error(error, GWY_MODULE_FILE_ERROR,
                     GWY_MODULE_FILE_ERROR_SPECIFIC,
@@ -214,19 +202,20 @@ nst_load(const gchar *filename,
         return NULL;
     }
 
-    container = gwy_container_new();
-    status = unzGoToFirstFile(zipfile);
-    while (status == UNZ_OK) {
-        unz_file_info fileinfo;
-        gchar filename_buf[PATH_MAX+1];
+    status = gwyzip_first_file(zipfile, error);
+    if (!status)
+        goto fail;
 
-        if (unzGetCurrentFileInfo(zipfile, &fileinfo, filename_buf,
-                                  PATH_MAX, NULL, 0, NULL, 0) != UNZ_OK) {
-            goto fail;
-        }
-        if (g_str_has_suffix(filename_buf, ".lsdlsd")) {
-            gwy_debug("channel %d: %s\n", channelno, filename_buf);
-            buffer = nst_get_file_content(zipfile, &size, NULL);
+    container = gwy_container_new();
+    while (status && gwyzip_get_current_filename(zipfile, &filename_curr,
+                                                 NULL)) {
+        if (g_str_has_suffix(filename_curr, ".lsdlsd")) {
+            gwy_debug("channel %d: %s\n", channelno, filename_curr);
+            buffer = gwyzip_get_file_content(zipfile, &size, error);
+            if (!buffer) {
+                g_free(filename_curr);
+                goto fail;
+            }
             p = buffer;
             line = gwy_str_next_line(&p);
             g_strstrip(line);
@@ -340,14 +329,16 @@ nst_load(const gchar *filename,
             g_free(buffer);
             channelno++;
         }
-        status = unzGoToNextFile(zipfile);
+        status = gwyzip_next_file(zipfile, NULL);
+        g_free(filename_curr);
     }
 
 fail:
-    unzClose(zipfile);
+    gwyzip_close(zipfile);
     if (!channelno) {
         gwy_object_unref(container);
-        err_NO_DATA(error);
+        if (error && !*error)
+            err_NO_DATA(error);
     }
 
     return container;
@@ -453,6 +444,51 @@ nst_read_3d(const gchar *buffer, GwyContainer **metadata, gchar **title)
                                            -1, "UTF-8", "cp1251",
                                            NULL, NULL, NULL);
                 }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "XYUnit")) {
+                    if ((!siunitxy) && (lineparts[linecur+1])) {
+                        unit = g_convert(lineparts[linecur+1], -1,
+                                         "UTF-8", "cp1251",
+                                         NULL, NULL, NULL);
+                        siunitxy = gwy_si_unit_new_parse(unit,
+                                                         &power10xy);
+                        g_free(unit);
+                    }
+                }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "ZUnit")) {
+                    if ((!siunitz) && (lineparts[linecur+1])) {
+                        unit = g_convert(lineparts[linecur+1], -1,
+                                         "UTF-8", "cp1251",
+                                         NULL, NULL, NULL);
+                        siunitz = gwy_si_unit_new_parse(unit, &power10z);
+                        g_free(unit);
+                    }
+                }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "XMin")) {
+                    if (lineparts[linecur+1])
+                        xoffset = g_ascii_strtod(lineparts[linecur+1],
+                                                 FALSE);
+                }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "XMax")) {
+                    if (lineparts[linecur+1])
+                        xscale = g_ascii_strtod(lineparts[linecur+1],
+                                                FALSE) - xoffset;
+                }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "YMin")) {
+                    if (lineparts[linecur+1])
+                        yoffset = g_ascii_strtod(lineparts[linecur+1],
+                                                 FALSE);
+                }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "YMax")) {
+                    if (lineparts[linecur+1])
+                        yscale = g_ascii_strtod(lineparts[linecur+1],
+                                                FALSE) - yoffset;
+                }
                 linecur++;
             }
             g_strfreev(lineparts);
@@ -509,13 +545,16 @@ static GwyGraphModel* nst_read_2d(const gchar *buffer, guint channel)
     p = (gchar *)buffer;
     gmodel = gwy_graph_model_new();
     while ((line = gwy_str_next_line(&p))) {
-        if (g_str_has_prefix(line, "Loved")) {
+        if (g_str_has_prefix(line, "[BeginOfItem]")) {
+            line = gwy_str_next_line(&p);
+            /* deprecated field check */
+            if (line && g_str_has_prefix(line, "Loved"))
+                line = gwy_str_next_line(&p);
+
             numpoints = 0;
             xarray = g_array_new(FALSE, TRUE, sizeof(gdouble));
             yarray = g_array_new(FALSE, TRUE, sizeof(gdouble));
-            while ((line = gwy_str_next_line(&p))
-                && (!gwy_strequal(line, "[EndOfItem]"))) {
-                                  lineparts = g_strsplit(line, " ", 3);
+            while (line && !gwy_strequal(line, "[EndOfItem]")) {
                 lineparts = g_strsplit(line, " ", 2);
                 x = g_ascii_strtod(lineparts[0], NULL);
                 g_array_append_val(xarray, x);
@@ -523,6 +562,8 @@ static GwyGraphModel* nst_read_2d(const gchar *buffer, guint channel)
                 g_array_append_val(yarray, y);
                 g_strfreev(lineparts);
                 numpoints++;
+
+                line = gwy_str_next_line(&p);
             }
 
             if (numpoints) {
@@ -613,6 +654,28 @@ static GwyGraphModel* nst_read_2d(const gchar *buffer, guint channel)
                                            -1, "UTF-8", "cp1251",
                                            NULL, NULL, NULL);
                 }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "XUnit")) {
+                    if ((!siunitx) && (lineparts[linecur+1])) {
+                        unit = g_convert(lineparts[linecur+1], -1,
+                                         "UTF-8", "cp1251",
+                                         NULL, NULL, NULL);
+                        siunitx = gwy_si_unit_new_parse(unit,
+                                                        &power10x);
+                        g_free(unit);
+                    }
+                }
+                else if (g_str_has_prefix(lineparts[linecur],
+                                                            "YUnit")) {
+                    if ((!siunity) && (lineparts[linecur+1])) {
+                        unit = g_convert(lineparts[linecur+1], -1,
+                                         "UTF-8", "cp1251",
+                                         NULL, NULL, NULL);
+                        siunity = gwy_si_unit_new_parse(unit,
+                                                        &power10y);
+                        g_free(unit);
+                    }
+                }
 
                 linecur++;
             }
@@ -862,50 +925,4 @@ exit:
     return brick;
 }
 
-
-static guchar*
-nst_get_file_content(unzFile *zipfile, gsize *contentsize, GError **error)
-{
-    unz_file_info fileinfo;
-    guchar *buffer;
-    gulong size;
-    glong readbytes;
-    gint status;
-
-    gwy_debug("calling unzGetCurrentFileInfo() to figure out buffer size");
-    status = unzGetCurrentFileInfo(zipfile, &fileinfo,
-                                   NULL, 0,
-                                   NULL, 0,
-                                   NULL, 0);
-    if (status != UNZ_OK) {
-        err_MINIZIP(status, error);
-        return NULL;
-    }
-
-    gwy_debug("calling unzGetCurrentFileInfo()");
-    status = unzOpenCurrentFile(zipfile);
-    if (status != UNZ_OK) {
-        err_MINIZIP(status, error);
-        return NULL;
-    }
-
-    size = fileinfo.uncompressed_size;
-    buffer = g_new(guchar, size + 1);
-    gwy_debug("calling unzReadCurrentFile()");
-    readbytes = unzReadCurrentFile(zipfile, buffer, size);
-    if (readbytes != size) {
-        err_MINIZIP(status, error);
-        unzCloseCurrentFile(zipfile);
-        g_free(buffer);
-        return NULL;
-    }
-    gwy_debug("calling unzCloseCurrentFile()");
-    unzCloseCurrentFile(zipfile);
-
-    buffer[size] = '\0';
-    if (contentsize)
-        *contentsize = size;
-    return buffer;
-}
-
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
