@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: rawfilepreset.h 16703 2014-10-19 09:19:10Z yeti-dn $
+ *  @(#) $Id: rawfilepreset.h 17529 2015-10-10 16:15:03Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -39,6 +39,8 @@ typedef enum {
     RAW_IEEE_DOUBLE,
     RAW_SIGNED_WORD64,
     RAW_UNSIGNED_WORD64,
+    RAW_IEEE_HALF,
+    RAW_PASCAL_REAL,
     RAW_LAST
 } RawFileBuiltin;
 
@@ -66,6 +68,10 @@ typedef struct {
     gchar *xyunit;
     gchar *zunit;
 
+    /* Missing values. */
+    gboolean havemissing;
+    gdouble missingvalue;
+
     /* Binary */
     RawFileBuiltin builtin;
     guint32 offset;  /* offset from file start, in bytes */
@@ -112,14 +118,15 @@ static const GwyRawFilePresetData rawfilepresetdata_default = {
     100.0, 100.0, -6,               /* physical dimensions */
     1.0, -6,                        /* z-scale */
     NULL, NULL,                     /* units */
+    FALSE, -32767.0,                /* missing values */
     RAW_UNSIGNED_BYTE, 0, 8, 0, 0,  /* binary parameters */
     FALSE, FALSE, FALSE, 0,         /* binary options */
     0, NULL, 0, FALSE,              /* text parameters */
 };
 
 /* sizes of RawFile built-in types */
-static const guint BUILTIN_SIZE[] = {
-    0, 8, 8, 16, 16, 32, 32, 32, 64,
+static const guint BUILTIN_SIZE[RAW_LAST] = {
+    0, 8, 8, 16, 16, 32, 32, 32, 64, 64, 64, 16, 48,
 };
 
 G_DEFINE_TYPE(GwyRawFilePreset, gwy_raw_file_preset, GWY_TYPE_RESOURCE)
@@ -177,6 +184,8 @@ gwy_raw_file_preset_data_sanitize(GwyRawFilePresetData *data)
     data->xyexponent = CLAMP(data->xyexponent, -12, 3);
     data->zexponent = CLAMP(data->zexponent, -12, 3);
 
+    data->havemissing = !!data->havemissing;
+
     if (!data->delimiter)
         data->delimiter = g_strdup("");
     if (!data->xyunit)
@@ -281,6 +290,16 @@ gwy_raw_file_preset_dump(GwyResource *resource,
         g_free(s);
     }
 
+    if (data->havemissing) {
+        /* Recycle the zscale string. */
+        g_ascii_dtostr(zscale, sizeof(zscale), data->missingvalue);
+        g_string_append_printf(str,
+                               "havemissing %d\n"
+                               "missingvalue %s\n",
+                               data->havemissing,
+                               zscale);
+    }
+
     /* Binary */
     g_string_append_printf(str,
                            "builtin %u\n"
@@ -384,6 +403,11 @@ gwy_raw_file_preset_parse(const gchar *text,
                 data.zunit = g_strcompress(value);
             }
         }
+        /* Missing values. */
+        else if (gwy_strequal(key, "havemissing"))
+            data.havemissing = atoi(value);
+        else if (gwy_strequal(key, "missingvalue"))
+            data.missingvalue = g_ascii_strtod(value, NULL);
         /* Binary */
         else if (gwy_strequal(key, "builtin"))
             data.builtin = atoi(value);
