@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: bdep_synth.c 17138 2015-05-10 06:17:15Z yeti-dn $
+ *  @(#) $Id: bdep_synth.c 17691 2015-11-01 10:27:46Z yeti-dn $
  *  Copyright (C) 2015 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -28,14 +28,11 @@
 #include <libprocess/stats.h>
 #include <libprocess/arithmetic.h>
 #include <libprocess/filters.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwydgets/gwydgetutils.h>
 #include <libgwydgets/gwystock.h>
 #include <libgwymodule/gwymodule-process.h>
 #include <app/gwyapp.h>
-
 #include "dimensions.h"
+#include "preview.h"
 
 #define BDEP_SYNTH_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -339,7 +336,6 @@ bdep_synth_dialog(BDepSynthArgs *args,
     GtkWidget *dialog, *table, *vbox, *hbox, *notebook, *hbox2, *check, *label;
     BDepSynthControls controls;
     GwyDataField *dfield;
-    GwyPixmapLayer *layer;
     gboolean finished;
     gint response;
     gint row, i;
@@ -379,14 +375,7 @@ bdep_synth_dialog(BDepSynthArgs *args,
                                                          PREVIEW_SIZE);
         controls.zscale = 3.0*gwy_data_field_get_rms(dfield_template);
     }
-    controls.view = gwy_data_view_new(controls.mydata);
-    layer = gwy_layer_basic_new();
-    g_object_set(layer,
-                 "data-key", "/0/data",
-                 "gradient-key", "/0/base/palette",
-                 NULL);
-    gwy_data_view_set_base_layer(GWY_DATA_VIEW(controls.view), layer);
-
+    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), controls.view, FALSE, FALSE, 0);
 
     hbox2 = gwy_synth_instant_updates_new(&controls,
