@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: fft_synth.c 16421 2014-07-22 12:23:26Z yeti-dn $
+ *  @(#) $Id: fft_synth.c 17691 2015-11-01 10:27:46Z yeti-dn $
  *  Copyright (C) 2007,2009,2010,2013 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -27,13 +27,10 @@
 #include <libprocess/stats.h>
 #include <libprocess/arithmetic.h>
 #include <libprocess/inttrans.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
-#include <libgwydgets/gwydgetutils.h>
 #include <libgwymodule/gwymodule-process.h>
 #include <app/gwyapp.h>
-
 #include "dimensions.h"
+#include "preview.h"
 
 #define FFT_SYNTH_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -337,7 +334,6 @@ fft_synth_dialog(FFTSynthArgs *args,
     GtkWidget *dialog, *table, *vbox, *hbox, *notebook, *spin;
     FFTSynthControls controls;
     GwyDataField *dfield;
-    GwyPixmapLayer *layer;
     gboolean finished;
     gint response;
     gint row;
@@ -377,14 +373,7 @@ fft_synth_dialog(FFTSynthArgs *args,
                                                          PREVIEW_SIZE);
         controls.zscale = gwy_data_field_get_rms(dfield_template);
     }
-    controls.view = gwy_data_view_new(controls.mydata);
-    layer = gwy_layer_basic_new();
-    g_object_set(layer,
-                 "data-key", "/0/data",
-                 "gradient-key", "/0/base/palette",
-                 NULL);
-    gwy_data_view_set_base_layer(GWY_DATA_VIEW(controls.view), layer);
-
+    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), controls.view, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(vbox),
