@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: threshold.c 16421 2014-07-22 12:23:26Z yeti-dn $
+ *  @(#) $Id: threshold.c 17691 2015-11-01 10:27:46Z yeti-dn $
  *  Copyright (C) 2010 David Necas (Yeti)
  *  E-mail: yeti@gwyddion.net
  *
@@ -25,13 +25,11 @@
 #include <libgwyddion/gwymath.h>
 #include <libprocess/filters.h>
 #include <libprocess/stats.h>
-#include <libgwydgets/gwydataview.h>
-#include <libgwydgets/gwylayer-basic.h>
 #include <libgwydgets/gwyradiobuttons.h>
-#include <libgwydgets/gwydgetutils.h>
 #include <libgwymodule/gwymodule-process.h>
 #include <app/gwymoduleutils.h>
 #include <app/gwyapp.h>
+#include "preview.h"
 
 #define THRESHOLD_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
@@ -237,7 +235,6 @@ threshold_dialog(ThresholdArgs *args,
     GtkTable *table;
     ThresholdControls controls;
     gint response;
-    GwyPixmapLayer *layer;
     gchar *s;
     gint row;
 
@@ -272,18 +269,7 @@ threshold_dialog(ThresholdArgs *args,
                             GWY_DATA_ITEM_RANGE,
                             GWY_DATA_ITEM_REAL_SQUARE,
                             0);
-    controls.view = gwy_data_view_new(controls.mydata);
-    layer = gwy_layer_basic_new();
-    g_object_set(layer,
-                 "data-key", "/0/data",
-                 "gradient-key", "/0/base/palette",
-                 "range-type-key", "/0/base/range-type",
-                 "min-max-key", "/0/base",
-                 NULL);
-    gwy_data_view_set_data_prefix(GWY_DATA_VIEW(controls.view), "/0/data");
-    gwy_data_view_set_base_layer(GWY_DATA_VIEW(controls.view), layer);
-    gwy_set_data_preview_size(GWY_DATA_VIEW(controls.view), PREVIEW_SIZE);
-
+    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
     gtk_box_pack_start(GTK_BOX(hbox), controls.view, FALSE, FALSE, 4);
 
     table = GTK_TABLE(gtk_table_new(5, 4, FALSE));
@@ -401,10 +387,10 @@ threshold_get_display_range(GwyContainer *container,
                             gdouble *disp_max)
 {
     GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
-    gchar key[64];
 
-    g_snprintf(key, sizeof(key), "/%d/base/range-type", id);
-    gwy_container_gis_enum_by_name(container, key, &range_type);
+    gwy_container_gis_enum(container,
+                           gwy_app_get_data_range_type_key_for_id(id),
+                           &range_type);
 
     switch (range_type) {
         case GWY_LAYER_BASIC_RANGE_FULL:
@@ -414,10 +400,12 @@ threshold_get_display_range(GwyContainer *container,
 
         case GWY_LAYER_BASIC_RANGE_FIXED:
         gwy_data_field_get_min_max(data_field, disp_min, disp_max);
-        g_snprintf(key, sizeof(key), "/%d/base/min", id);
-        gwy_container_gis_double_by_name(container, key, disp_min);
-        g_snprintf(key, sizeof(key), "/%d/base/max", id);
-        gwy_container_gis_double_by_name(container, key, disp_max);
+        gwy_container_gis_double(container,
+                                 gwy_app_get_data_range_min_key_for_id(id),
+                                 disp_min);
+        gwy_container_gis_double(container,
+                                 gwy_app_get_data_range_min_key_for_id(id),
+                                 disp_max);
         break;
 
         case GWY_LAYER_BASIC_RANGE_AUTO:
