@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: tip_blind.c 17385 2015-09-14 07:32:37Z yeti-dn $
+ *  @(#) $Id: tip_blind.c 17671 2015-10-31 09:44:39Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -932,7 +932,6 @@ tip_blind_do_images(TipBlindControls *controls,
     data = gwy_app_data_browser_get(args->source.datano);
     for (i = 0; i < args->nstripes; i++) {
         gchar *title;
-        gchar key[24];
 
         if (!args->goodtip[i] || !args->stripetips[i])
             continue;
@@ -943,8 +942,9 @@ tip_blind_do_images(TipBlindControls *controls,
                                 GWY_DATA_ITEM_GRADIENT, 0);
         title = g_strdup_printf("%s %u/%u",
                                 _("Estimated tip"), i+1, args->nstripes);
-        g_snprintf(key, sizeof(key), "/%d/data/title", newid);
-        gwy_container_set_string_by_name(data, key, title);
+        gwy_container_set_string(data,
+                                 gwy_app_get_data_title_key_for_id(newid),
+                                 title);
         gwy_app_channel_log_add_proc(data, -1, newid);
     }
 
