@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: filter.c 16974 2015-03-30 13:16:06Z yeti-dn $
+ *  @(#) $Id: filter.c 17704 2015-11-03 22:35:37Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -24,10 +24,13 @@
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libgwymodule/gwymodule-tool.h>
+#include <libprocess/gwyprocesstypes.h>
+#include <libprocess/elliptic.h>
 #include <libprocess/filters.h>
 #include <libprocess/linestats.h>
 #include <libgwydgets/gwystock.h>
 #include <libgwydgets/gwycombobox.h>
+#include <libgwydgets/gwyradiobuttons.h>
 #include <libgwydgets/gwydgetutils.h>
 #include <app/gwyapp.h>
 
@@ -40,22 +43,28 @@
 
 /* Keep numbering for settings */
 typedef enum {
-    GWY_FILTER_MEAN          = 0,
-    GWY_FILTER_MEDIAN        = 1,
-    GWY_FILTER_CONSERVATIVE  = 2,
-    GWY_FILTER_MINIMUM       = 3,
-    GWY_FILTER_MAXIMUM       = 4,
-    GWY_FILTER_KUWAHARA      = 5,
-    GWY_FILTER_DECHECKER     = 6,
-    GWY_FILTER_GAUSSIAN      = 7,
-    GWY_FILTER_SHARPEN       = 8
-} GwyFilterType;
+    FILTER_MEAN          = 0,
+    FILTER_MEDIAN        = 1,
+    FILTER_CONSERVATIVE  = 2,
+    FILTER_MINIMUM       = 3,
+    FILTER_MAXIMUM       = 4,
+    FILTER_KUWAHARA      = 5,
+    FILTER_DECHECKER     = 6,
+    FILTER_GAUSSIAN      = 7,
+    FILTER_SHARPEN       = 8,
+    FILTER_OPENING       = 9,
+    FILTER_CLOSING       = 10,
+    FILTER_ASF_OPENING   = 11,
+    FILTER_ASF_CLOSING   = 12,
+    FILTER_NFILTERS
+} FilterType;
 
 typedef struct _GwyToolFilter      GwyToolFilter;
 typedef struct _GwyToolFilterClass GwyToolFilterClass;
 
 typedef struct {
-    GwyFilterType filter_type;
+    FilterType filter_type;
+    GwyMaskingType masking;
     gint size;
     gdouble gauss_size;
 } ToolArgs;
@@ -69,6 +78,7 @@ struct _GwyToolFilter {
     GtkWidget *filter_type;
     GtkObject *size;
     GtkWidget *size_spin;
+    GSList *masking;
     GtkWidget *apply;
 
     /* potential class data */
@@ -79,8 +89,7 @@ struct _GwyToolFilterClass {
     GwyPlainToolClass parent_class;
 };
 
-static gboolean module_register(void);
-
+static gboolean module_register                  (void);
 static GType    gwy_tool_filter_get_type         (void)                      G_GNUC_CONST;
 static void     gwy_tool_filter_finalize         (GObject *object);
 static void     gwy_tool_filter_init_dialog      (GwyToolFilter *tool);
@@ -95,10 +104,22 @@ static void     gwy_tool_filter_size_changed     (GwyToolFilter *tool,
                                                   GtkAdjustment *adj);
 static void     gwy_tool_filter_type_changed     (GtkComboBox *combo,
                                                   GwyToolFilter *tool);
-static gboolean gwy_tool_filter_is_sized         (GwyFilterType type);
-static gboolean gwy_tool_filter_is_float_sized   (GwyFilterType type);
+static void     gwy_tool_filter_masking_changed  (GtkWidget *button,
+                                                  GwyToolFilter *tool);
+static gboolean gwy_tool_filter_is_sized         (FilterType type);
+static gboolean gwy_tool_filter_is_float_sized   (FilterType type);
 static void     setup_size_adjustment            (GwyToolFilter *tool);
 static void     gwy_tool_filter_apply            (GwyToolFilter *tool);
+static void     filter_area_sharpen              (GwyDataField *dfield,
+                                                  gdouble sigma,
+                                                  gint col,
+                                                  gint row,
+                                                  gint width,
+                                                  gint height);
+static void     apply_masking                    (GwyDataField *dfield,
+                                                  GwyDataField *orig,
+                                                  GwyDataField *mask,
+                                                  GwyMaskingType masking);
 static void     gwy_tool_filter_save_args        (GwyToolFilter *tool);
 
 static GwyModuleInfo module_info = {
@@ -107,17 +128,19 @@ static GwyModuleInfo module_info = {
     N_("Filter tool, processes selected part of data with a filter "
        "(conservative denoise, mean, median. Kuwahara, minimum, maximum)."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "3.9",
+    "3.11",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
 
 static const gchar filter_type_key[] = "/module/filter/filter_type";
-static const gchar size_key[]        = "/module/filter/size";
 static const gchar gauss_size_key[]  = "/module/filter/gauss_size";
+static const gchar masking_key[]     = "/module/filter/masking";
+static const gchar size_key[]        = "/module/filter/size";
 
 static const ToolArgs default_args = {
-    GWY_FILTER_MEAN,
+    FILTER_MEAN,
+    GWY_MASK_IGNORE,
     5,
     5.0,
 };
@@ -179,11 +202,17 @@ gwy_tool_filter_init(GwyToolFilter *tool)
     tool->args = default_args;
     gwy_container_gis_enum_by_name(settings, filter_type_key,
                                    &tool->args.filter_type);
+    gwy_container_gis_enum_by_name(settings, masking_key,
+                                   &tool->args.masking);
     gwy_container_gis_int32_by_name(settings, size_key,
                                     &tool->args.size);
     gwy_container_gis_double_by_name(settings, gauss_size_key,
                                      &tool->args.gauss_size);
 
+    tool->args.filter_type = MIN(tool->args.filter_type, FILTER_NFILTERS-1);
+    tool->args.masking = gwy_enum_sanitize_value(tool->args.masking,
+                                                 GWY_TYPE_MASKING_TYPE);
+
     gwy_plain_tool_connect_selection(plain_tool, tool->layer_type_rect,
                                      "rectangle");
 
@@ -205,15 +234,19 @@ static void
 gwy_tool_filter_init_dialog(GwyToolFilter *tool)
 {
     static const GwyEnum filters[] = {
-        { N_("Mean value"),           GWY_FILTER_MEAN,         },
-        { N_("Median value"),         GWY_FILTER_MEDIAN,       },
-        { N_("Conservative denoise"), GWY_FILTER_CONSERVATIVE, },
-        { N_("Minimum"),              GWY_FILTER_MINIMUM,      },
-        { N_("Maximum"),              GWY_FILTER_MAXIMUM,      },
-        { N_("Kuwahara"),             GWY_FILTER_KUWAHARA,     },
-        { N_("Dechecker"),            GWY_FILTER_DECHECKER,    },
-        { N_("filter|Gaussian"),      GWY_FILTER_GAUSSIAN,     },
-        { N_("Sharpen"),              GWY_FILTER_SHARPEN,      },
+        { N_("Mean value"),           FILTER_MEAN,         },
+        { N_("Median value"),         FILTER_MEDIAN,       },
+        { N_("Conservative denoise"), FILTER_CONSERVATIVE, },
+        { N_("Minimum"),              FILTER_MINIMUM,      },
+        { N_("Maximum"),              FILTER_MAXIMUM,      },
+        { N_("filter|Opening"),       FILTER_OPENING,      },
+        { N_("filter|Closing"),       FILTER_CLOSING,      },
+        { N_("ASF Opening"),          FILTER_ASF_OPENING,  },
+        { N_("ASF Closing"),          FILTER_ASF_CLOSING,  },
+        { N_("Kuwahara"),             FILTER_KUWAHARA,     },
+        { N_("Dechecker"),            FILTER_DECHECKER,    },
+        { N_("filter|Gaussian"),      FILTER_GAUSSIAN,     },
+        { N_("Sharpen"),              FILTER_SHARPEN,      },
     };
     GtkDialog *dialog;
     GtkTable *table;
@@ -263,6 +296,19 @@ gwy_tool_filter_init_dialog(GwyToolFilter *tool)
                              G_CALLBACK(gwy_tool_filter_size_changed), tool);
     row++;
 
+    gtk_table_set_row_spacing(table, row-1, 8);
+    label = gwy_label_new_header(_("Masking Mode"));
+    gtk_table_attach(table, label,
+                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    row++;
+
+    tool->masking
+        = gwy_radio_buttons_create(gwy_masking_type_get_enum(), -1,
+                                   G_CALLBACK(gwy_tool_filter_masking_changed),
+                                   tool,
+                                   tool->args.masking);
+    row = gwy_radio_buttons_attach_to_table(tool->masking, table, 3, row);
+
     gwy_plain_tool_add_clear_button(GWY_PLAIN_TOOL(tool));
     gwy_tool_add_hide_button(GWY_TOOL(tool), FALSE);
     tool->apply = gtk_dialog_add_button(dialog, GTK_STOCK_APPLY,
@@ -359,7 +405,7 @@ static void
 gwy_tool_filter_type_changed(GtkComboBox *combo,
                              GwyToolFilter *tool)
 {
-    GwyFilterType prevtype, newtype;
+    FilterType prevtype, newtype;
     gboolean sensitive, prevfloat, newfloat;
 
     prevtype = tool->args.filter_type;
@@ -373,18 +419,37 @@ gwy_tool_filter_type_changed(GtkComboBox *combo,
         setup_size_adjustment(tool);
 }
 
-static gboolean
-gwy_tool_filter_is_float_sized(GwyFilterType type)
+static void
+gwy_tool_filter_masking_changed(GtkWidget *button,
+                                GwyToolFilter *tool)
 {
-    return (type == GWY_FILTER_GAUSSIAN
-            || type == GWY_FILTER_SHARPEN);
+    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
+        return;
+
+    tool->args.masking = gwy_radio_button_get_value(button);
 }
 
 static gboolean
-gwy_tool_filter_is_sized(GwyFilterType type)
+gwy_tool_filter_is_float_sized(FilterType type)
 {
-    return (type != GWY_FILTER_KUWAHARA
-            && type != GWY_FILTER_DECHECKER);
+    return (type == FILTER_GAUSSIAN
+            || type == FILTER_SHARPEN);
+}
+
+static gboolean
+gwy_tool_filter_is_sized(FilterType type)
+{
+    return (type != FILTER_KUWAHARA
+            && type != FILTER_DECHECKER);
+}
+
+static gboolean
+gwy_tool_filter_needs_kernel(FilterType type)
+{
+    return (type == FILTER_MINIMUM
+            || type == FILTER_MAXIMUM
+            || type == FILTER_OPENING
+            || type == FILTER_CLOSING);
 }
 
 static void
@@ -397,11 +462,166 @@ setup_size_adjustment(GwyToolFilter *tool)
         gtk_spin_button_set_digits(GTK_SPIN_BUTTON(tool->size_spin), 2);
     }
     else {
-        gtk_adjustment_configure(adj, tool->args.size, 2, 20, 1, 5, 0);
+        gtk_adjustment_configure(adj, tool->args.size, 2, 31, 1, 5, 0);
         gtk_spin_button_set_digits(GTK_SPIN_BUTTON(tool->size_spin), 0);
     }
 }
 
+static void
+gwy_tool_filter_apply(GwyToolFilter *tool)
+{
+    GwyPlainTool *plain_tool;
+    GwyDataField *dfield, *origfield = NULL, *kernel = NULL;
+    gdouble sel[4];
+    gint isel[4];
+    gdouble sigma;
+    gint size;
+
+    plain_tool = GWY_PLAIN_TOOL(tool);
+    size = tool->args.size;
+    sigma = tool->args.gauss_size*FWHM2SIGMA;
+    dfield = plain_tool->data_field;
+    g_return_if_fail(plain_tool->id >= 0 && dfield != NULL);
+    gwy_tool_filter_save_args(tool);
+
+    if (gwy_selection_get_object(plain_tool->selection, 0, sel)) {
+        isel[0] = floor(gwy_data_field_rtoj(dfield, sel[0]));
+        isel[1] = floor(gwy_data_field_rtoi(dfield, sel[1]));
+        isel[2] = floor(gwy_data_field_rtoj(dfield, sel[2]));
+        isel[3] = floor(gwy_data_field_rtoi(dfield, sel[3]));
+
+        if (sel[0] > sel[2])
+            GWY_SWAP(gdouble, sel[0], sel[2]);
+        if (sel[1] > sel[3])
+            GWY_SWAP(gdouble, sel[1], sel[3]);
+        isel[2] -= isel[0] + 1;
+        isel[3] -= isel[1] + 1;
+    }
+    else {
+        isel[0] = isel[1] = 0;
+        isel[2] = gwy_data_field_get_xres(dfield);
+        isel[3] = gwy_data_field_get_yres(dfield);
+    }
+
+    gwy_app_undo_qcheckpoint(plain_tool->container,
+                             gwy_app_get_data_key_for_id(plain_tool->id), 0);
+
+    if (gwy_tool_filter_needs_kernel(tool->args.filter_type)) {
+        kernel = gwy_data_field_new(size, size, size, size, TRUE);
+        gwy_data_field_elliptic_area_fill(kernel, 0, 0, size, size, 1.0);
+    }
+
+    /*
+     * Remember the original for merging when masking is used.
+     * XXX: This is inefficient when the area to actually modify is small.
+     * However, linear operations are implemented using FFT and most
+     * morphological operations are implemented using some moving window
+     * algorithms.  For these we would have to switch to an idependent
+     * pixel-by-pixel evaluation method (based on some ad-hoc threshold).
+     * Kuwahara, dechecker, conservative denoising and (currently) median
+     * are evaluated pixel-by-pixel anyway so native masking would not be
+     * difficult, but apart from median probably also not worth it.
+     */
+    if (tool->args.masking != GWY_MASK_IGNORE && plain_tool->mask_field)
+        origfield = gwy_data_field_duplicate(dfield);
+
+    switch (tool->args.filter_type) {
+        case FILTER_MEAN:
+        gwy_data_field_area_filter_mean(dfield, size,
+                                        isel[0], isel[1],
+                                        isel[2], isel[3]);
+        break;
+
+        case FILTER_MEDIAN:
+        gwy_data_field_area_filter_median(dfield, size,
+                                          isel[0], isel[1],
+                                          isel[2], isel[3]);
+        break;
+
+        case FILTER_MINIMUM:
+        gwy_data_field_area_filter_min_max(dfield, kernel,
+                                           GWY_MIN_MAX_FILTER_MINIMUM,
+                                           isel[0], isel[1],
+                                           isel[2], isel[3]);
+        break;
+
+        case FILTER_MAXIMUM:
+        gwy_data_field_area_filter_min_max(dfield, kernel,
+                                           GWY_MIN_MAX_FILTER_MAXIMUM,
+                                           isel[0], isel[1],
+                                           isel[2], isel[3]);
+        break;
+
+        case FILTER_CONSERVATIVE:
+        gwy_data_field_area_filter_conservative(dfield, size,
+                                                isel[0], isel[1],
+                                                isel[2], isel[3]);
+        break;
+
+        case FILTER_KUWAHARA:
+        gwy_data_field_area_filter_kuwahara(dfield,
+                                            isel[0], isel[1],
+                                            isel[2], isel[3]);
+        break;
+
+        case FILTER_DECHECKER:
+        gwy_data_field_area_filter_dechecker(dfield,
+                                             isel[0], isel[1],
+                                             isel[2], isel[3]);
+        break;
+
+        case FILTER_GAUSSIAN:
+        gwy_data_field_area_filter_gaussian(dfield, sigma,
+                                            isel[0], isel[1],
+                                            isel[2], isel[3]);
+        break;
+
+        case FILTER_SHARPEN:
+        filter_area_sharpen(dfield, sigma, isel[0], isel[1], isel[2], isel[3]);
+        break;
+
+        case FILTER_OPENING:
+        gwy_data_field_area_filter_min_max(dfield, kernel,
+                                           GWY_MIN_MAX_FILTER_OPENING,
+                                           isel[0], isel[1],
+                                           isel[2], isel[3]);
+        break;
+
+        case FILTER_CLOSING:
+        gwy_data_field_area_filter_min_max(dfield, kernel,
+                                           GWY_MIN_MAX_FILTER_CLOSING,
+                                           isel[0], isel[1],
+                                           isel[2], isel[3]);
+        break;
+
+        case FILTER_ASF_OPENING:
+        gwy_data_field_area_filter_disc_asf(dfield, size/2, FALSE,
+                                            isel[0], isel[1],
+                                            isel[2], isel[3]);
+        break;
+
+        case FILTER_ASF_CLOSING:
+        gwy_data_field_area_filter_disc_asf(dfield, size/2, TRUE,
+                                            isel[0], isel[1],
+                                            isel[2], isel[3]);
+        break;
+
+        default:
+        g_assert_not_reached();
+        break;
+    }
+
+    if (origfield) {
+        apply_masking(dfield, origfield,
+                      plain_tool->mask_field, tool->args.masking);
+        gwy_object_unref(origfield);
+    }
+
+    gwy_object_unref(kernel);
+    gwy_data_field_data_changed(dfield);
+    gwy_plain_tool_log_add(plain_tool);
+}
+
 static void
 filter_area_sharpen(GwyDataField *dfield, gdouble sigma,
                     gint col, gint row, gint width, gint height)
@@ -426,107 +646,30 @@ filter_area_sharpen(GwyDataField *dfield, gdouble sigma,
 }
 
 static void
-gwy_tool_filter_apply(GwyToolFilter *tool)
+apply_masking(GwyDataField *dfield, GwyDataField *orig, GwyDataField *mask,
+              GwyMaskingType masking)
 {
-    GwyPlainTool *plain_tool;
-    gdouble sel[4];
-    gint isel[4];
+    const gdouble *r = gwy_data_field_get_data_const(orig);
+    const gdouble *m = gwy_data_field_get_data_const(mask);
+    gdouble *d = gwy_data_field_get_data(dfield);
+    gint xres = gwy_data_field_get_xres(dfield);
+    gint yres = gwy_data_field_get_yres(dfield);
+    gint k;
 
-    plain_tool = GWY_PLAIN_TOOL(tool);
-    g_return_if_fail(plain_tool->id >= 0 && plain_tool->data_field != NULL);
-    gwy_tool_filter_save_args(tool);
-
-    if (gwy_selection_get_object(plain_tool->selection, 0, sel)) {
-        isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
-        isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
-        isel[2] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[2]));
-        isel[3] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[3]));
-
-        if (sel[0] > sel[2])
-            GWY_SWAP(gdouble, sel[0], sel[2]);
-        if (sel[1] > sel[3])
-            GWY_SWAP(gdouble, sel[1], sel[3]);
-        isel[2] -= isel[0] + 1;
-        isel[3] -= isel[1] + 1;
+    if (masking == GWY_MASK_INCLUDE) {
+        for (k = 0; k < xres*yres; k++) {
+            if (m[k] <= 0.0)
+                d[k] = r[k];
+        }
     }
     else {
-        isel[0] = isel[1] = 0;
-        isel[2] = gwy_data_field_get_xres(plain_tool->data_field);
-        isel[3] = gwy_data_field_get_yres(plain_tool->data_field);
+        for (k = 0; k < xres*yres; k++) {
+            if (m[k] > 0.0)
+                d[k] = r[k];
+        }
     }
 
-    gwy_app_undo_qcheckpoint(plain_tool->container,
-                             gwy_app_get_data_key_for_id(plain_tool->id), 0);
-
-    switch (tool->args.filter_type) {
-        case GWY_FILTER_MEAN:
-        gwy_data_field_area_filter_mean(plain_tool->data_field,
-                                        tool->args.size,
-                                        isel[0], isel[1],
-                                        isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_MEDIAN:
-        gwy_data_field_area_filter_median(plain_tool->data_field,
-                                          tool->args.size,
-                                          isel[0], isel[1],
-                                          isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_MINIMUM:
-        gwy_data_field_area_filter_minimum(plain_tool->data_field,
-                                           tool->args.size,
-                                           isel[0], isel[1],
-                                           isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_MAXIMUM:
-        gwy_data_field_area_filter_maximum(plain_tool->data_field,
-                                           tool->args.size,
-                                           isel[0], isel[1],
-                                           isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_CONSERVATIVE:
-        gwy_data_field_area_filter_conservative(plain_tool->data_field,
-                                                tool->args.size,
-                                                isel[0], isel[1],
-                                                isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_KUWAHARA:
-        gwy_data_field_area_filter_kuwahara(plain_tool->data_field,
-                                            isel[0], isel[1],
-                                            isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_DECHECKER:
-        gwy_data_field_area_filter_dechecker(plain_tool->data_field,
-                                             isel[0], isel[1],
-                                             isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_GAUSSIAN:
-        gwy_data_field_area_filter_gaussian(plain_tool->data_field,
-                                            tool->args.gauss_size*FWHM2SIGMA,
-                                            isel[0], isel[1],
-                                            isel[2], isel[3]);
-        break;
-
-        case GWY_FILTER_SHARPEN:
-        filter_area_sharpen(plain_tool->data_field,
-                            tool->args.gauss_size*FWHM2SIGMA,
-                            isel[0], isel[1],
-                            isel[2], isel[3]);
-        break;
-
-        default:
-        g_assert_not_reached();
-        break;
-    }
-
-    gwy_data_field_data_changed(plain_tool->data_field);
-    gwy_plain_tool_log_add(plain_tool);
+    gwy_data_field_invalidate(dfield);
 }
 
 static void
@@ -537,6 +680,7 @@ gwy_tool_filter_save_args(GwyToolFilter *tool)
     settings = gwy_app_settings_get();
     gwy_container_set_enum_by_name(settings, filter_type_key,
                                    tool->args.filter_type);
+    gwy_container_set_enum_by_name(settings, masking_key, tool->args.masking);
     gwy_container_set_int32_by_name(settings, size_key,
                                     tool->args.size);
     gwy_container_set_double_by_name(settings, gauss_size_key,
