@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: icolorange.c 17222 2015-07-30 12:16:58Z yeti-dn $
+ *  @(#) $Id: icolorange.c 17709 2015-11-05 09:00:30Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -69,7 +69,9 @@ struct _GwyToolColorRange {
     GtkLabel *max;
     GtkLabel *datamin;
     GtkLabel *datamax;
-    GtkWidget *mask_hbox;
+    GtkWidget *set_to_masked;
+    GtkWidget *set_to_unmasked;
+    GtkWidget *invert;
 
     ColorRangeSource range_source;
     gboolean programmatic_update;
@@ -118,6 +120,7 @@ static void   gwy_tool_color_range_update_histogram (GwyToolColorRange *tool);
 static void   gwy_tool_color_range_spin_changed     (GwyToolColorRange *tool);
 static void   set_range_to_masked                   (GwyToolColorRange *tool);
 static void   set_range_to_unmasked                 (GwyToolColorRange *tool);
+static void   invert_mapping                        (GwyToolColorRange *tool);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -126,7 +129,7 @@ static GwyModuleInfo module_info = {
        "color scale should map to, either on data or on height distribution "
        "histogram."),
     "Yeti <yeti@gwyddion.net>",
-    "3.15",
+    "3.16",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -247,6 +250,7 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
     GtkDialog *dialog;
     GwyGraphCurveModel *cmodel;
     GwyGraphArea *garea;
+    GtkSizeGroup *sizegroup;
     GwyLayerBasicRangeType range_type = GWY_LAYER_BASIC_RANGE_FULL;
     GtkTooltips *tips;
     gint row, i;
@@ -314,7 +318,7 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
                        TRUE, TRUE, 2);
 
     /* Data ranges */
-    table = GTK_TABLE(gtk_table_new(7, 3, FALSE));
+    table = GTK_TABLE(gtk_table_new(8, 3, FALSE));
     gtk_table_set_col_spacings(table, 6);
     gtk_table_set_row_spacings(table, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), 4);
@@ -377,21 +381,40 @@ gwy_tool_color_range_init_dialog(GwyToolColorRange *tool)
                      1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
     row++;
 
-    tool->mask_hbox = gtk_hbox_new(FALSE, 0);
-    gtk_table_attach(table, tool->mask_hbox,
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_table_attach(table, hbox,
                      0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
+    sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+
     button = gtk_button_new_with_mnemonic(_("Set to _Unmasked"));
-    gtk_box_pack_end(GTK_BOX(tool->mask_hbox), button, FALSE, TRUE, 0);
+    tool->set_to_unmasked = button;
+    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
+    gtk_size_group_add_widget(sizegroup, button);
     g_signal_connect_swapped(button, "clicked",
                              G_CALLBACK(set_range_to_unmasked), tool);
 
     button = gtk_button_new_with_mnemonic(_("Set to _Masked"));
-    gtk_box_pack_end(GTK_BOX(tool->mask_hbox), button, FALSE, TRUE, 0);
+    tool->set_to_masked = button;
+    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
+    gtk_size_group_add_widget(sizegroup, button);
     g_signal_connect_swapped(button, "clicked",
                              G_CALLBACK(set_range_to_masked), tool);
     row++;
 
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_table_attach(table, hbox,
+                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+
+    button = gtk_button_new_with_mnemonic(_("_Invert Mapping"));
+    tool->invert = button;
+    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
+    gtk_size_group_add_widget(sizegroup, button);
+    g_signal_connect_swapped(button, "clicked",
+                             G_CALLBACK(invert_mapping), tool);
+    g_object_unref(sizegroup);
+    row++;
+
     gtk_table_set_row_spacing(table, row-1, 8);
     label = gwy_label_new_header(gwy_sgettext("range|Full"));
     gtk_table_attach(table, label, 0, 1, row, row+1,
@@ -543,7 +566,6 @@ gwy_tool_color_range_data_changed(GwyPlainTool *plain_tool)
                                    plain_tool->data_field,
                                    NULL, NULL);
     gwy_tool_color_range_update_histogram(GWY_TOOL_COLOR_RANGE(plain_tool));
-    gwy_tool_color_range_selection_changed(plain_tool, 0);
 }
 
 static void
@@ -552,9 +574,9 @@ gwy_tool_color_range_mask_changed(GwyPlainTool *plain_tool)
     GwyToolColorRange *tool = GWY_TOOL_COLOR_RANGE(plain_tool);
     gboolean have_mask = !!plain_tool->mask_field;
     gboolean range_type = gwy_tool_color_range_get_range_type(tool);
-    gtk_widget_set_sensitive(tool->mask_hbox,
-                             have_mask
-                             && range_type == GWY_LAYER_BASIC_RANGE_FIXED);
+    gboolean msens = (have_mask && range_type == GWY_LAYER_BASIC_RANGE_FIXED);
+    gtk_widget_set_sensitive(tool->set_to_masked, msens);
+    gtk_widget_set_sensitive(tool->set_to_unmasked, msens);
 }
 
 static void
@@ -656,6 +678,7 @@ gwy_tool_color_range_type_changed(GtkWidget *radio,
         gtk_widget_set_sensitive(GTK_WIDGET(tool->histogram), fixed);
         gtk_widget_set_sensitive(GTK_WIDGET(tool->spinmin), fixed);
         gtk_widget_set_sensitive(GTK_WIDGET(tool->spinmax), fixed);
+        gtk_widget_set_sensitive(GTK_WIDGET(tool->invert), fixed);
 
         gwy_tool_color_range_set_range_type(tool, range_type);
         if (fixed && !tool->data_switch) {
@@ -951,5 +974,24 @@ set_range_to_unmasked(GwyToolColorRange *tool)
     set_range_using_mask(tool, GWY_MASK_EXCLUDE);
 }
 
+static void
+invert_mapping(GwyToolColorRange *tool)
+{
+    gdouble min, max;
+
+    if (!GWY_PLAIN_TOOL(tool)->data_field)
+        return;
+
+    min = gtk_spin_button_get_value(GTK_SPIN_BUTTON(tool->spinmin));
+    max = gtk_spin_button_get_value(GTK_SPIN_BUTTON(tool->spinmax));
+
+    tool->programmatic_update = TRUE;
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(tool->spinmin), max);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(tool->spinmax), min);
+    tool->programmatic_update = FALSE;
+
+    gwy_tool_color_range_spin_changed(tool);
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
 
