@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: linestats.c 17139 2015-05-10 08:32:36Z yeti-dn $
+ *  @(#) $Id: linestats.c 17461 2015-09-25 11:32:17Z yeti-dn $
  *  Copyright (C) 2003-2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -134,7 +134,7 @@ static GwyModuleInfo module_info = {
     N_("Row/column statistical function tool, mean values, medians, maxima, "
        "minima, RMS, ..., of rows or columns."),
     "Yeti <yeti@gwyddion.net>",
-    "1.5",
+    "1.6",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -163,9 +163,11 @@ static const GwyEnum sf_types[] =  {
     { N_("Median"),             GWY_LINE_STAT_MEDIAN,    },
     { N_("Minimum"),            GWY_LINE_STAT_MINIMUM,   },
     { N_("Maximum"),            GWY_LINE_STAT_MAXIMUM,   },
+    { N_("Range"),              GWY_LINE_STAT_RANGE,     },
     { N_("Length"),             GWY_LINE_STAT_LENGTH,    },
     { N_("Slope"),              GWY_LINE_STAT_SLOPE,     },
     { N_("tan β<sub>0</sub>"),  GWY_LINE_STAT_TAN_BETA0, },
+    { N_("Variation"),          GWY_LINE_STAT_VARIATION, },
     { N_("Ra"),                 GWY_LINE_STAT_RA,        },
     { N_("Rq (RMS)"),           GWY_LINE_STAT_RMS,       },
     { N_("Rz"),                 GWY_LINE_STAT_RZ,        },
@@ -493,7 +495,6 @@ gwy_tool_line_stats_data_switched(GwyTool *gwytool,
     }
 
     gwy_tool_line_stats_update_curve(tool);
-    gwy_tool_line_stats_update_target_graphs(tool);
 }
 
 static void
@@ -514,7 +515,6 @@ gwy_tool_line_stats_data_changed(GwyPlainTool *plain_tool)
 {
     GwyToolLineStats *tool = GWY_TOOL_LINE_STATS(plain_tool);
     gwy_tool_line_stats_update_curve(tool);
-    gwy_tool_line_stats_update_target_graphs(tool);
 }
 
 static void
@@ -642,6 +642,7 @@ gwy_tool_line_stats_update_curve(GwyToolLineStats *tool)
     g_object_set(gcmodel, "description", title, NULL);
     g_object_set(tool->gmodel, "title", title, NULL);
     gwy_graph_model_set_units_from_data_line(tool->gmodel, tool->line);
+    gwy_tool_line_stats_update_target_graphs(tool);
 
     siunit = gwy_data_line_get_si_unit_y(tool->line);
     format = gwy_si_unit_get_format(siunit, GWY_SI_UNIT_FORMAT_MARKUP,
@@ -688,7 +689,6 @@ gwy_tool_line_stats_output_type_changed(GtkComboBox *combo,
     tool->args.output_type = gwy_enum_combo_box_get_active(combo);
     gwy_tool_line_stats_update_sensitivity(tool);
     gwy_tool_line_stats_update_curve(tool);
-    gwy_tool_line_stats_update_target_graphs(tool);
 }
 
 static void
