@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: profile.c 17461 2015-09-25 11:32:17Z yeti-dn $
+ *  @(#) $Id: profile.c 17718 2015-11-06 08:04:04Z pecold $
  *  Copyright (C) 2003-2015 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -19,7 +19,6 @@
  *  Boston, MA 02110-1301, USA.
  */
 
-#include <stdio.h>
 #include "config.h"
 #include <stdlib.h>
 #include <gtk/gtk.h>
@@ -222,7 +221,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Profile tool, creates profile graphs from selected lines."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "3.2",
+    "3.3",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -1421,6 +1420,12 @@ get_profile_with_masking(GwyDataField *dfield,
         res = GWY_ROUND(size);
 
     gwy_debug("size: %g, res: %d", size, res);
+    if (xto == xfrom && yto == yfrom) {
+        xto += 0.2*dx;
+        yto += 0.2*dy;
+        xfrom -= 0.2*dx;
+        yfrom -= 0.2*dy;
+    }
     xstep = (xto - xfrom)/(res - 1.0);
     ystep = (yto - yfrom)/(res - 1.0);
     step = hypot(xstep, ystep);
