@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: stats.c 17258 2015-08-07 19:43:13Z yeti-dn $
+ *  @(#) $Id: stats.c 17718 2015-11-06 08:04:04Z pecold $
  *  Copyright (C) 2003-2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -21,9 +21,7 @@
 
 #include "config.h"
 #include <string.h>
-#include <stdio.h>
 #include <errno.h>
-#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
@@ -276,8 +274,8 @@ gwy_tool_stats_init(GwyToolStats *tool)
     gwy_container_gis_boolean_by_name(settings, instant_update_key,
                                       &tool->args.instant_update);
 
-    tool->args.masking
-        = gwy_enum_sanitize_value(tool->args.masking, GWY_TYPE_MASKING_TYPE);
+    tool->args.masking = gwy_enum_sanitize_value(tool->args.masking,
+                                                 GWY_TYPE_MASKING_TYPE);
 
     tool->angle_format = g_new0(GwySIValueFormat, 1);
     tool->angle_format->magnitude = 1.0;
