@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: volume_kmeans.c 17038 2015-04-15 20:35:40Z dn2010 $
+ *  @(#) $Id: volume_kmeans.c 17482 2015-09-28 08:21:44Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek,
  *  Daniil Bratashov, Evgeniy Ryabov.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, dn2010@gmail.com.
@@ -21,6 +21,7 @@
  */
 
 #include "config.h"
+#include <string.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libprocess/arithmetic.h>
@@ -48,6 +49,7 @@ typedef struct {
 } KMeansArgs;
 
 typedef struct {
+    KMeansArgs *args;
     GtkObject *k;
     GtkObject *epsilon;
     GtkObject *max_iterations;
@@ -59,8 +61,6 @@ static void      volume_kmeans       (GwyContainer *data,
                                       GwyRunType run);
 static void      kmeans_dialog       (GwyContainer *data,
                                       KMeansArgs *args);
-static void      epsilon_changed_cb  (GtkAdjustment *adj,
-                                      KMeansArgs *args);
 static void      kmeans_dialog_update(KMeansControls *controls,
                                       KMeansArgs *args);
 static void      kmeans_values_update(KMeansControls *controls,
@@ -86,7 +86,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Calculates K-means clustering on volume data."),
     "Daniil Bratashov <dn2010@gmail.com> & Evgeniy Ryabov",
-    "1.2",
+    "1.3",
     "David Nečas (Yeti) & Petr Klapetek & Daniil Bratashov & Evgeniy Ryabov",
     "2014",
 };
@@ -115,6 +115,7 @@ volume_kmeans(GwyContainer *data, GwyRunType run)
     gint id;
 
     g_return_if_fail(run & KMEANS_RUN_MODES);
+    g_return_if_fail(data);
 
     kmeans_load_args(gwy_app_settings_get(), &args);
     gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
@@ -123,6 +124,7 @@ volume_kmeans(GwyContainer *data, GwyRunType run)
     g_return_if_fail(GWY_IS_BRICK(brick));
     if (run == GWY_RUN_INTERACTIVE) {
         kmeans_dialog(data, &args);
+        kmeans_save_args(gwy_app_settings_get(), &args);
     }
     else if (run == GWY_RUN_IMMEDIATE) {
         volume_kmeans_do(data, &args);
@@ -130,25 +132,23 @@ volume_kmeans(GwyContainer *data, GwyRunType run)
 }
 
 static void
-kmeans_dialog (GwyContainer *data, KMeansArgs *args)
+kmeans_dialog(GwyContainer *data, KMeansArgs *args)
 {
-    GtkWidget *dialog, *table, *spin;
+    GtkWidget *dialog, *table;
     gint response;
     KMeansControls controls;
     gint row = 0;
 
-    dialog = gtk_dialog_new_with_buttons(_("K-means"),
-                                         NULL, 0, NULL);
+    controls.args = args;
+
+    dialog = gtk_dialog_new_with_buttons(_("K-means"), NULL, 0, NULL);
     gtk_dialog_add_button(GTK_DIALOG(dialog),
                           _("_Reset"), RESPONSE_RESET);
     gtk_dialog_add_button(GTK_DIALOG(dialog),
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-    gtk_dialog_add_action_widget(GTK_DIALOG(dialog),
-                                 gwy_stock_like_button_new(_("_Run"),
-                                                          GTK_STOCK_OK),
-                                 GTK_RESPONSE_OK);
-    gtk_dialog_set_default_response(GTK_DIALOG(dialog),
-                                    GTK_RESPONSE_CANCEL);
+    gtk_dialog_add_button(GTK_DIALOG(dialog),
+                          GTK_STOCK_OK, GTK_RESPONSE_OK);
+    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     gwy_help_add_to_volume_dialog(GTK_DIALOG(dialog), GWY_HELP_DEFAULT);
 
     table = gtk_table_new(4, 4, FALSE);
@@ -159,30 +159,23 @@ kmeans_dialog (GwyContainer *data, KMeansArgs *args)
                        TRUE, TRUE, 4);
 
     controls.k = gtk_adjustment_new(args->k, 2, 100, 1, 10, 0);
-    spin = gwy_table_attach_spinbutton(table, row,
-                                       _("_Number of clusters:"), "",
-                                       controls.k);
-    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
+    gwy_table_attach_hscale(table, row,
+                            _("_Number of clusters:"), NULL,
+                            controls.k, GWY_HSCALE_SQRT);
     row++;
 
-    controls.epsilon = gtk_adjustment_new(args->epsilon,
-                                          1e-20, 0.1, 1, 10, 0);
-    spin = gwy_table_attach_hscale(table, row,
-                                   _("Convergence _Precision:"), NULL,
-                                   controls.epsilon, GWY_HSCALE_LOG);
-    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 20);
-    g_object_set_data(G_OBJECT(controls.epsilon),
-                      "controls", &controls);
-    g_signal_connect(controls.epsilon, "value-changed",
-                     G_CALLBACK(epsilon_changed_cb), args);
+    controls.epsilon = gtk_adjustment_new(-log10(args->epsilon),
+                                          1.0, 20.0, 0.01, 1.0, 0);
+    gwy_table_attach_hscale(table, row,
+                            _("Convergence _precision digits:"), NULL,
+                            controls.epsilon, GWY_HSCALE_DEFAULT);
     row++;
 
     controls.max_iterations = gtk_adjustment_new(args->max_iterations,
                                                  1, 10000, 1, 1, 0);
-    spin = gwy_table_attach_spinbutton(table, row,
-                                       _("_Max. iterations:"), "",
-                                       controls.max_iterations);
-    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
+    gwy_table_attach_hscale(table, row,
+                            _("_Max. iterations:"), NULL,
+                            controls.max_iterations, GWY_HSCALE_LOG);
     row++;
 
     controls.normalize
@@ -198,44 +191,32 @@ kmeans_dialog (GwyContainer *data, KMeansArgs *args)
         switch (response) {
             case GTK_RESPONSE_CANCEL:
             case GTK_RESPONSE_DELETE_EVENT:
-                kmeans_values_update(&controls, args);
-                gtk_widget_destroy(dialog);
+            kmeans_values_update(&controls, args);
+            gtk_widget_destroy(dialog);
             case GTK_RESPONSE_NONE:
             return;
             break;
 
             case GTK_RESPONSE_OK:
-                kmeans_values_update(&controls, args);
-                volume_kmeans_do(data, args);
             break;
 
             case RESPONSE_RESET:
-                *args = kmeans_defaults;
-                kmeans_dialog_update(&controls, args);
+            *args = kmeans_defaults;
+            kmeans_dialog_update(&controls, args);
             break;
 
             default:
-                g_assert_not_reached();
+            g_assert_not_reached();
             break;
         }
     } while (response != GTK_RESPONSE_OK);
 
     kmeans_values_update(&controls, args);
-    kmeans_save_args(gwy_app_settings_get(), args);
     gtk_widget_destroy(dialog);
+    volume_kmeans_do(data, args);
 }
 
-static void
-epsilon_changed_cb(GtkAdjustment *adj,
-                   KMeansArgs *args)
-{
-    KMeansControls *controls;
-
-    controls = g_object_get_data(G_OBJECT(adj), "controls");
-    args->epsilon = gtk_adjustment_get_value(adj);
-    kmeans_dialog_update(controls, args);
-}
-
+/* XXX: Duplicate with volume_kmedians.c */
 static GwyBrick*
 normalize_brick(GwyBrick *brick, GwyDataField *intfield)
 {
@@ -255,7 +236,7 @@ normalize_brick(GwyBrick *brick, GwyDataField *intfield)
     newdata = gwy_brick_get_data(result);
     intdata = gwy_data_field_get_data(intfield);
 
-    for (i = 0; i < xres; i++)
+    for (i = 0; i < xres; i++) {
         for (j = 0; j < yres; j++) {
             integral = 0;
             for (l = 0; l < zres; l++) {
@@ -296,6 +277,7 @@ normalize_brick(GwyBrick *brick, GwyDataField *intfield)
             }
             *(intdata + j * xres + i) = integral / zres;
         }
+    }
 
     return result;
 }
@@ -304,26 +286,22 @@ static void
 kmeans_values_update(KMeansControls *controls,
                      KMeansArgs *args)
 {
-    args->k
-        = gtk_adjustment_get_value(GTK_ADJUSTMENT(controls->k));
-    args->epsilon
-        = gtk_adjustment_get_value(GTK_ADJUSTMENT(controls->epsilon));
+    args->k = gwy_adjustment_get_int(GTK_ADJUSTMENT(controls->k));
+    args->epsilon = gtk_adjustment_get_value(GTK_ADJUSTMENT(controls->epsilon));
+    args->epsilon = pow(0.1, args->epsilon);
     args->max_iterations
-        = gtk_adjustment_get_value(
-                              GTK_ADJUSTMENT(controls->max_iterations));
+        = gwy_adjustment_get_int(GTK_ADJUSTMENT(controls->max_iterations));
     args->normalize
-        = gtk_toggle_button_get_active(
-                                GTK_TOGGLE_BUTTON(controls->normalize));
+        = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(controls->normalize));
 }
 
 static void
 kmeans_dialog_update(KMeansControls *controls,
                      KMeansArgs *args)
 {
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->k),
-                             args->k);
+    gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->k), args->k);
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->epsilon),
-                             args->epsilon);
+                             -log10(args->epsilon));
     gtk_adjustment_set_value(GTK_ADJUSTMENT(controls->max_iterations),
                              args->max_iterations);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->normalize),
@@ -353,7 +331,7 @@ volume_kmeans_do(GwyContainer *container, KMeansArgs *args)
     gint k = args->k;
     gint iterations = 0;
     gint max_iterations = args->max_iterations;
-    gboolean converged = FALSE;
+    gboolean converged = FALSE, cancelled = FALSE;
     gboolean normalize = args->normalize;
 
     gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
@@ -377,13 +355,14 @@ volume_kmeans_do(GwyContainer *container, KMeansArgs *args)
 
     siunit = gwy_brick_get_si_unit_x(brick);
     gwy_data_field_set_si_unit_xy(dfield, siunit);
-    siunit = gwy_si_unit_new(_("Cluster"));
-    gwy_data_field_set_si_unit_z(dfield, siunit);
 
     intmap = gwy_data_field_new_alike(dfield, TRUE);
     siunit = gwy_brick_get_si_unit_w(brick);
     gwy_data_field_set_si_unit_z(intmap, siunit);
 
+    gwy_app_wait_start(gwy_app_find_window_for_volume(container, id),
+                       _("Initializing..."));
+
     if (normalize) {
         normalized = normalize_brick(brick, intmap);
         data = gwy_brick_get_data_const(normalized);
@@ -392,13 +371,13 @@ volume_kmeans_do(GwyContainer *container, KMeansArgs *args)
         data = gwy_brick_get_data_const(brick);
     }
 
-    centers = g_malloc(zres*k*sizeof(gdouble));
-    oldcenters = g_malloc (zres*k*sizeof(gdouble));
-    sum = g_malloc(zres*k*sizeof(gdouble));
-    npix = g_malloc(k*sizeof(gint));
+    centers = g_new(gdouble, zres*k);
+    oldcenters = g_new(gdouble, zres*k);
+    sum = g_new(gdouble, zres*k);
+    npix = g_new(gint, k);
     data1 = gwy_data_field_get_data(dfield);
 
-    rand=g_rand_new();
+    rand = g_rand_new();
     for (c = 0; c < k; c++) {
         i = g_rand_int_range(rand, 0, xres);
         j = g_rand_int_range(rand, 0, yres);
@@ -409,7 +388,15 @@ volume_kmeans_do(GwyContainer *container, KMeansArgs *args)
     };
     g_rand_free(rand);
 
-    while (!converged) {
+    if (!gwy_app_wait_set_message(_("K-means iteration...")))
+        cancelled = TRUE;
+
+    while (!converged && !cancelled) {
+        if (!gwy_app_wait_set_fraction((gdouble)iterations/max_iterations)) {
+            cancelled = TRUE;
+            break;
+        }
+
         /* pixels belong to cluster with min distance */
         for (j = 0; j < yres; j++)
             for (i = 0; i < xres; i++) {
@@ -440,7 +427,7 @@ volume_kmeans_do(GwyContainer *container, KMeansArgs *args)
                 *(sum + c * zres + l) = 0;
             }
         }
-        for (i = 0; i < xres; i++)
+        for (i = 0; i < xres; i++) {
             for (j = 0; j < yres; j++) {
                 c = (gint)(*(data1 + j * xres + i));
                 *(npix + c) += 1;
@@ -449,137 +436,145 @@ volume_kmeans_do(GwyContainer *container, KMeansArgs *args)
                             += *(data + l * xres * yres + j * xres + i);
                 }
             }
-
+        }
         for (c = 0; c < k; c++)
-            for (l =0; l < zres; l++) {
+            for (l = 0; l < zres; l++) {
                 *(centers + c * zres + l) = (*(npix + c) > 0) ?
-                     *(sum + c * zres + l) / (gdouble)(*(npix + c)) : 0;
+                     *(sum + c * zres + l)/(gdouble)(*(npix + c)) : 0.0;
         }
 
         converged = TRUE;
-        for (c = 0; c < k; c++)
+        for (c = 0; c < k; c++) {
             for (l = 0; l < zres; l++)
                 if (fabs(*(oldcenters + c * zres + l)
                                - *(centers + c * zres + l)) > epsilon) {
                     converged = FALSE;
                     break;
                 }
+        }
         if (iterations == max_iterations) {
             converged = TRUE;
         }
         iterations++;
     }
 
-    if (container) {
-        errormap = gwy_data_field_new_alike(dfield, TRUE);
-        siunit = gwy_si_unit_new(_("Error"));
+    gwy_app_wait_finish();
+    if (cancelled)
+        goto fail;
+
+    errormap = gwy_data_field_new_alike(dfield, TRUE);
+    if (!normalize) {
+        siunit = gwy_brick_get_si_unit_w(brick);
+        siunit = gwy_si_unit_duplicate(siunit);
         gwy_data_field_set_si_unit_z(errormap, siunit);
-        errordata = gwy_data_field_get_data(errormap);
+        g_object_unref(siunit);
+    }
+    errordata = gwy_data_field_get_data(errormap);
 
-        for (i = 0; i < xres; i++)
-            for (j = 0; j < yres; j++) {
-                dist = 0.0;
-                c = (gint)(*(data1 + j * xres + i));
-                for (l = 0; l < zres; l++) {
-                    dist += (*(data + l * xres * yres + j * xres + i)
-                           - *(centers + c * zres + l))
-                          * (*(data + l * xres * yres + j * xres + i)
-                           - *(centers + c * zres + l));
-                }
-                *(errordata + j * xres + i) = sqrt(dist);
+    for (i = 0; i < xres; i++) {
+        for (j = 0; j < yres; j++) {
+            dist = 0.0;
+            c = (gint)(*(data1 + j * xres + i));
+            for (l = 0; l < zres; l++) {
+                dist += (*(data + l * xres * yres + j * xres + i)
+                         - *(centers + c * zres + l))
+                    * (*(data + l * xres * yres + j * xres + i)
+                       - *(centers + c * zres + l));
             }
+            *(errordata + j * xres + i) = sqrt(dist);
+        }
+    }
 
-        gwy_data_field_add(dfield, 1.0);
-        newid = gwy_app_data_browser_add_data_field(dfield,
+    gwy_data_field_add(dfield, 1.0);
+    newid = gwy_app_data_browser_add_data_field(dfield,
+                                                container, TRUE);
+    gwy_object_unref(dfield);
+    description = gwy_app_get_brick_title(container, id);
+    gwy_app_set_data_field_title(container, newid,
+                                 g_strdup_printf(_("K-means cluster of %s"),
+                                                 description));
+    gwy_app_channel_log_add(container, -1, newid, "volume::kmeans",
+                            NULL);
+
+    newid = gwy_app_data_browser_add_data_field(errormap,
+                                                container, TRUE);
+    gwy_object_unref(errormap);
+    gwy_app_set_data_field_title(container, newid,
+                                 g_strdup_printf(_("K-means error of %s"),
+                                                 description));
+    gwy_app_channel_log_add(container, -1, newid, "volume::kmeans",
+                            NULL);
+
+    if (normalize) {
+        newid = gwy_app_data_browser_add_data_field(intmap,
                                                     container, TRUE);
-        g_object_unref(dfield);
-        description = gwy_app_get_brick_title(container, id);
+        gwy_object_unref(intmap);
         gwy_app_set_data_field_title(container, newid,
-                                     g_strdup_printf(_("K-means of %s"),
-                                                     description)
-                                     );
-        gwy_app_channel_log_add(container, -1, newid, "volume::kmeans",
-                                NULL);
+                                     g_strdup_printf(_("Pre-normalized "
+                                                       "intensity of %s"),
+                                                     description));
 
-        newid = gwy_app_data_browser_add_data_field(errormap,
-                                                    container, TRUE);
-        g_object_unref(errormap);
-        gwy_app_set_data_field_title(container, newid,
-                                     g_strdup_printf(
-                                               _("K-means error of %s"),
-                                               description)
-                                     );
-        gwy_app_channel_log_add(container, -1, newid, "volume::kmeans",
-                                NULL);
+        gwy_app_channel_log_add(container, -1, newid,
+                                "volume::kmeans", NULL);
+    }
 
-        if (normalize) {
-            newid = gwy_app_data_browser_add_data_field(intmap,
-                                                        container, TRUE);
-            g_object_unref(intmap);
-            gwy_app_set_data_field_title(container, newid,
-                    g_strdup_printf(
-                                    _("Pre-normalized intensity of %s"),
-                                    description)
-                                    );
+    g_free(description);
 
-            gwy_app_channel_log_add(container, -1, newid,
-                                    "volume::kmeans", NULL);
-        }
-
-        g_free(description);
-
-        gmodel = gwy_graph_model_new();
-        calibration = gwy_brick_get_zcalibration(brick);
-        if (calibration) {
-            xdata = gwy_data_line_get_data(calibration);
-            siunit = gwy_data_line_get_si_unit_y(calibration);
-        }
-        else {
-            xdata = g_malloc(zres * sizeof(gdouble));
-            for (i = 0; i < zres; i++)
-                *(xdata + i) = zreal * i / zres + zoffset;
-            siunit = gwy_brick_get_si_unit_z(brick);
-        }
-        for (c = 0; c < k; c++) {
-            ydata = g_memdup(centers + c * zres,
-                             zres * sizeof(gdouble));
-            gcmodel = gwy_graph_curve_model_new();
-            gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, zres);
-            rgba = gwy_graph_get_preset_color(c);
-            g_object_set(gcmodel,
-                         "mode", GWY_GRAPH_CURVE_LINE,
-                         "description",
-                         g_strdup_printf(_("K-means center %d"), c + 1),
-                         "color", rgba,
-                         NULL);
-            gwy_graph_model_add_curve(gmodel, gcmodel);
-            g_object_unref(gcmodel);
-        }
-        g_object_set(gmodel,
-                     "si-unit-x", siunit,
-                     "si-unit-y", gwy_brick_get_si_unit_w(brick),
-                     "axis-label-bottom", "x",
-                     "axis-label-left", "y",
+    gmodel = gwy_graph_model_new();
+    calibration = gwy_brick_get_zcalibration(brick);
+    xdata = g_new(gdouble, zres);
+    ydata = g_new(gdouble, zres);
+    if (calibration) {
+        memcpy(xdata, gwy_data_line_get_data(calibration),
+               zres*sizeof(gdouble));
+        siunit = gwy_data_line_get_si_unit_y(calibration);
+    }
+    else {
+        for (i = 0; i < zres; i++)
+            *(xdata + i) = zreal * i / zres + zoffset;
+        siunit = gwy_brick_get_si_unit_z(brick);
+    }
+    for (c = 0; c < k; c++) {
+        memcpy(ydata, centers + c * zres, zres * sizeof(gdouble));
+        gcmodel = gwy_graph_curve_model_new();
+        gwy_graph_curve_model_set_data(gcmodel, xdata, ydata, zres);
+        rgba = gwy_graph_get_preset_color(c);
+        g_object_set(gcmodel,
+                     "mode", GWY_GRAPH_CURVE_LINE,
+                     "description",
+                     g_strdup_printf(_("K-means center %d"), c + 1),
+                     "color", rgba,
                      NULL);
-        gwy_app_data_browser_add_graph_model(gmodel, container, TRUE);
-        g_object_unref(gmodel);
+        gwy_graph_model_add_curve(gmodel, gcmodel);
+        g_object_unref(gcmodel);
     }
+    g_free(xdata);
+    g_free(ydata);
+    g_object_set(gmodel,
+                 "si-unit-x", siunit,
+                 "si-unit-y", gwy_brick_get_si_unit_w(brick),
+                 "axis-label-bottom", "x",
+                 "axis-label-left", "y",
+                 NULL);
+    gwy_app_data_browser_add_graph_model(gmodel, container, TRUE);
+    g_object_unref(gmodel);
 
-    if (normalized) {
-        g_object_unref(normalized);
-    }
+    gwy_app_volume_log_add_volume(container, id, id);
+
+fail:
+    gwy_object_unref(errormap);
+    gwy_object_unref(intmap);
+    gwy_object_unref(dfield);
+    gwy_object_unref(normalized);
     g_free(npix);
     g_free(sum);
     g_free(oldcenters);
     g_free(centers);
-
-    gwy_app_volume_log_add_volume(container, id, id);
 }
 
-static const gchar kmeans_k_key[]       = "/module/kmeans/k";
 static const gchar epsilon_key[]        = "/module/kmeans/epsilon";
-static const gchar max_iterations_key[]
-                                      = "/module/kmeans/max_iterations";
+static const gchar kmeans_k_key[]       = "/module/kmeans/k";
+static const gchar max_iterations_key[] = "/module/kmeans/max_iterations";
 static const gchar normalize_key[]      = "/module/kmeans/normalize";
 
 static void
@@ -589,7 +584,6 @@ kmeans_sanitize_args(KMeansArgs *args)
     args->epsilon = CLAMP(args->epsilon, 1e-20, 0.1);
     args->max_iterations = CLAMP(args->max_iterations, 0, 10000);
     args->normalize = !!args->normalize;
-
 }
 
 static void
@@ -599,8 +593,7 @@ kmeans_load_args(GwyContainer *container,
     *args = kmeans_defaults;
 
     gwy_container_gis_int32_by_name(container, kmeans_k_key, &args->k);
-    gwy_container_gis_double_by_name(container, epsilon_key,
-                                                        &args->epsilon);
+    gwy_container_gis_double_by_name(container, epsilon_key, &args->epsilon);
     gwy_container_gis_int32_by_name(container, max_iterations_key,
                                                  &args->max_iterations);
     gwy_container_gis_boolean_by_name(container, normalize_key,
@@ -614,12 +607,11 @@ kmeans_save_args(GwyContainer *container,
                  KMeansArgs *args)
 {
     gwy_container_set_int32_by_name(container, kmeans_k_key, args->k);
-    gwy_container_set_double_by_name(container, epsilon_key,
-                                                         args->epsilon);
+    gwy_container_set_double_by_name(container, epsilon_key, args->epsilon);
     gwy_container_set_int32_by_name(container, max_iterations_key,
                                                   args->max_iterations);
     gwy_container_set_boolean_by_name(container, normalize_key,
                                                        args->normalize);
 }
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
 
+/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
